<?php
/**
* PostgreSQL database access classes
*
* @package papaya_base
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: pgsql.php,v 1.11 2006/06/06 11:35:42 weinert Exp $
*/

/**
* basic database connection and result class
*/
require_once(dirname(__FILE__).'/base.php');

/**
* DB-abstraction layer - connection object PostgreSQL
*
* @package papaya_base
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class dbcon_pgsql extends dbcon_base {
  
  function extensionFound() {
    return (bool)extension_loaded('pgsql');
  }
  
  /**
  * Establish connection to database
  *
  * @return resource $this->databaseConnection connection ID
  */
  function connect() {
    if (!(isset($this->databaseConnection) && is_resource($this->databaseConnection))) {
      if (is_numeric($this->databaseConfiguration['port'])) {
        $server = 'host='.$this->databaseConfiguration['hostspec'];
        $port = ' port='.$this->databaseConfiguration['port'];
      } else {
        $server = 'host='.$this->databaseConfiguration['hostspec'];
        $port = '';
      }
      $connectStr = $server.' user='.$this->databaseConfiguration['username'].
                              ' password='.$this->databaseConfiguration['password'].
                              ' dbname='.$this->databaseConfiguration['database'].
                              $port;
      if (!($this->databaseConnection = pg_connect($connectStr))) {
        unset($this->databaseConnection);
      }
      if (isset($this->databaseConnection) && is_resource($this->databaseConnection)) {
        pg_query("SET CLIENT_ENCODING TO 'LATIN1'");
        return TRUE;
      } else {
        unset($this->databaseConnection);
        return FALSE;
      }
    } else {
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Provides error messages of last database connection
  *
  * @return array error number, error message and last sql query
  */
  function getError() {
    if (isset($this->databaseConnection) && is_resource($this->databaseConnection)) {
      return array('no'=>NULL, 'str'=>pg_last_error($this->databaseConnection), 'lastquery'=>$this->lastSQLQuery);
    } else {
      return array('no'=>NULL, 'str'=> NULL, 'lastquery'=>$this->lastSQLQuery);
    }
  }

  /**
  * Return an explanation of the last SELECT-SQL-query
  *
  * Explan the last query, number of lines etc.
  *
  * @return mixed Result or FALSE, if last request wasn't a SELECT query
  */  function getExplain() {
    if (preg_match('/^SELECT\s/i', $this->lastSQLQuery)) {
      if ($res = pg_query($this->databaseConnection, 'EXPLAIN  '.$this->lastSQLQuery)) {
        $result = '<ul>';
        while($row = pg_fetch_row($res)) {
          $result .= '<li>';
          $result .= (isset($row[0])) ? papaya_strings::escapeHTMLChars($row[0]) : '';
          $result .= '</li>';
        }
        $result .= '</ul>';
        return $result;
      }
    }
    return FALSE;
  }


  /**
  * String ecsaping for PostgreSQL use
  *
  * @param mixed $value Value to escape
  * @return string escaped value.
  */
  function escapeStr($value) {
    return pg_escape_string($value);
  }

  /**
  * Execute PostgreSQL-query
  *
  * @param string $sql SQL-String with query
  * @param integer $max maximum number of returned records
  * @param integer $offset Offset
  * @param boolean $debug Debug Query?
  * @return mixed FALSE or number of affected_rows or database result object
  */
  function &query($sql, $max=NULL, $offset=NULL, $debug = FALSE) {
    if (@get_class($this->lastResult) == 'dbresult_postgres') {
      $this->lastResult->free();
    }
    $absCount = 0;
    if (isset($max) && strpos(trim($sql), 'SELECT') === 0) {
      if (preg_match('~^\s*(SELECT.*)(\bFROM\b.*?)(\bORDER\s+BY.*)?$~si', $sql, $regs)) {
        $countSql = 'SELECT COUNT(*) '.$regs[2];
        if ($res = @pg_query($this->databaseConnection, $countSql)) {
          if ($row = @pg_fetch_row($res)) {
            $absCount = $row[0];
          }
          @pg_free_result($res);
        }
      }
      $sql .= ((isset($offset)) ? ' OFFSET '.(int)$offset : '').' LIMIT '.(int)$max;
    }
    $this->lastSQLQuery = $sql;

    $res = @pg_query($this->databaseConnection, $sql);

    if ($res) {
      if (@is_resource($res)) {
        $this->lastResult = &new dbresult_pgsql($res);
        $this->lastResult->setLimit($max, $offset);
        $this->lastResult->_absCount = $absCount;
        return $this->lastResult;
      } else {
        $result = @pg_affected_rows($this->databaseConnection);
        return $result;
      }
    } else {
      $result = FALSE;
      return $result;
    }
  }

  /**
  * Insert record into table
  *
  * @param string $table table
  * @param string $idField primary key value
  * @param array $values insert values
  * @return mixed FALSE or Id of new record
  */
  function insertRecord($table, $idField, $values = NULL) {
    if (isset($idField)) {
      $values[$idField] = NULL;
    }
    if (isset($values) && is_array($values) && count($values) > 0) {
      $fieldString = '';
      $valueString = '';
      foreach($values as $field=>$value) {
        if (is_bool($value)) {
          $fieldString .= $this->escapeStr($field).', ';
          $valueString .= "'".($value ? 1 : 0)."', ";
        } elseif ($value !== NULL) {
          $fieldString .= $this->escapeStr($field).', ';
          $valueString .= "'".$this->escapeStr($value)."', ";
        }
      }
      $fieldString = substr($fieldString, 0, -2);
      $valueString = substr($valueString, 0, -2);
      $sql = 'INSERT INTO '.$this->escapeStr($table).' ('.$fieldString.') VALUES ('.$valueString.')';
      if ($this->query($sql)) {
        if (isset($idField)) {
          $sql = 'SELECT MAX('.$this->escapeStr($idField).') FROM '.$this->escapeStr($table);
          if ($res = $this->query($sql)) {
            list($new) = $res->fetchRow();
            return $new;
          }
        } else {
          return @pg_affected_rows($this->databaseConnection);
        }
      }
    }
    return FALSE;
  }


  function insertRecords($table, $values) {
    $baseSQL = "COPY ".$this->escapeStr($table)." ";
    $lastFields = NULL;
    $specialChars = array("\t"=>'\t', "\r"=>'\r', "\n"=>'\n');
    if (isset($values) && is_array($values) && count($values) > 0) {
      foreach($values as $data) {
        if (is_array($data) && count($data) > 0) {
          $fields = array();
          $valueData = array();
          foreach($data as $key=>$val) {
            $fields[] = strtr($this->escapeStr($key), $specialChars);
            if ($val === '') {
              $valueData[] = '';
            } else {
              $valueData[] = strtr($this->escapeStr($val), $specialChars);
            }
          }
          if (!isset($lastFields)) {
            $sql = $baseSQL."(".implode(",", $fields).") FROM STDIN USING DELIMITERS '\t' WITH NULL AS '\\NULL' \n";
            if (!pg_query($sql)) {
              return FALSE;
            }
            $lastFields = $fields;
          } elseif (count(array_diff($fields, $lastFields)) > 0) {
            if (!pg_end_copy($this->databaseConnection)) {
              return FALSE;
            }
            $sql = $baseSQL."(".implode(",", $fields).") FROM STDIN USING DELIMITERS '\t' WITH NULL AS '\\NULL' \n";
            if (!pg_query($sql)) {
              return FALSE;
            }
            $lastFields = $fields;
          }
          $line = implode("\t", $valueData).LF;
          if (!pg_put_line($this->databaseConnection, $line)) {
            pg_end_copy($this->databaseConnection);
            return FALSE;
          }
        }
      }
      if (!pg_end_copy($this->databaseConnection)) {
        return FALSE;
      }
      $this->updateAutoIncrementFields();
      return TRUE;
    }
    return FALSE;
  }
  
  function updateAutoIncrementFields($table) {
    list(,$fields) = $this->queryTableStructure($table);
    if (isset($fields) && is_array($fields)) {
      foreach($fields as $field) {
        if ($field['autoinc'] == 'yes') {
          $tableName = $this->escapeString($table);
          $fieldName = $this->escapeString($field['name']);
          $sql = "\$max = SELECT MAX(".$fieldName.") FROM ".$tableName.";".LF;
          $sql .= "SELECT SETVAL(".$tableName."_seq) \$max);".LF;
          $this->query($sql);
        }
      }
    }
  }

  /**
  * Update records via filter
  *
  * @param string $table table name
  * @param array $values update values
  * @param string $filter Filter string without WHERE condition
  * @return mixed FALSE or number of affected_rows or database result object
  */
  function updateRecord($table, $values, $filter) {
    if (isset($values) && is_array($values) && count($values) > 0) {
      $sql = FALSE;
      foreach($values as $col=>$val) {
        $fieldName = trim($col);
        $sql .= " $fieldName='".$this->escapeStr($val)."', ";
      }
      if ($sql) {
        $sql = "UPDATE $table SET ".substr($sql, 0, -2)." WHERE ".$this->getSQLCondition($filter);
        return $this->query($sql);
      } else {
        $this->lastSQLQuery = 'NO DATA';
      }
    } else {
      $this->lastSQLQuery = '';
    }
    return FALSE;
  }

  /**
  * Delete records by filter
  *
  * @param string $table table name
  * @param string $filter Filter string without WHERE condition
  * @return mixed FALSE or number of affected_rows or database result object
  */
  function deleteRecord($table, $filter) {
    $sql = "DELETE FROM $table WHERE ".$this->getSQLCondition($filter);
    return $this->query($sql);
  }

  /**
  * Get table names
  *
  * @access public
  * @return array
  */
  function queryTableNames() {
    $sql = "SELECT tablename FROM pg_tables WHERE schemaname = 'public'";
    $result = array();
    if ($res = $this->query($sql)) {
      while($row = $res->fetchRow()) {
        $result[] = $row[0];
      }
    }
    return $result;
  }

  /**
  * Parse PostgreSQL field data
  *
  * @param array &$row
  * @access private
  * @return array
  */
  function parsePostgresFieldData(&$row) {
    $size = 0;
    $autoIncrement = 0;

    if (substr($row['fieldtype'], 0, 3) == 'int') {
      $type = 'integer';
      switch ($row['format_type']) {
        case 'bigint' :
          $size = 8;
          break;
        case 'integer' :
          $size = 4;
          break;
        case 'smallint' :
          $size = 2;
          break;
        default :
          $size = 8;
      }
    } elseif ($row['fieldtype'] == 'varchar' || $row['fieldtype'] == 'bpchar') {
      $type = 'string';
      $size = substr($row['format_type'], strrpos($row['format_type'], '(')+1, -1);
    } elseif ($row['fieldtype'] == 'text') {
      $type = 'string';
      $size = 65535;
    } else {
      $size = 16777215;
      $type = 'string';
    }

    if (preg_match('#^nextval\(\'[\w\.]+\'::text\)$#i', $row['defaultwert'], $regs)) {
      $autoIncrement = 1;
      $default = 1;
    } elseif (preg_match('#^\'(([^\']+|\\\\)+)\'::(character varying|bpchar)$#i', $row['defaultwert'], $regs)) {
      if ($regs[1] != '') {
        $default = $regs[1];
      }
    } elseif (preg_match('#^(\d+)::(smallint|integer|bigint|int)$#i', $row['defaultwert'], $regs)) {
      if ($regs[1] != 0) {
        $default = $regs[1];
      }
    }

    $result = array(
      'name' => $row['fieldname'],
      'type' => $type,
      'size' => $size,
      'null' => strtolower($row['not_null'] == 't') ? 'no' : 'yes',
      'autoinc' => strtolower($autoIncrement) ? 'yes' : 'no',
      'default' => NULL
    );

    if ((!$autoIncrement) && isset($default)) {
      $result['default'] = $default;
    }

    return $result;
  }

  /**
  * PostgresSQL field type
  *
  * @param string $type
  * @param string $size
  * @access private
  * @return string
  */
  function getPostgresFieldType(&$field, $seperated = FALSE) {
    if (isset($field['autoinc']) && $field['autoinc'] == 'yes') {
      $autoIncrementField = TRUE;
      if ($seperated) {
        $result['type'] = ($field['size'] > 4) ? 'BIGINT' : 'INTEGER';
        $result['not_null'] = 'NOT NULL';
        $result['default'] = 'DEFAULT 1';
        $result['autoinc'] = TRUE;
        return $result;
      } else {
        return ($field['size'] > 4) ? 'BIGSERIAL' : 'SERIAL';
      }
    } else {
      if (isset($field['null']) && $field['null'] == 'yes') {
        $default = NULL;
        $notNullStr = '';
      } else {
        $default = '';
        $notNullStr = ' NOT NULL';
      }
      if (isset($field['default'])) {
        $default = $field['default'];
      }
      $defaultStr = '';
      if (isset($default)) {
        switch(strtolower($field['type'])) {
          case 'integer' :
            $defaultStr = " DEFAULT '".(int)$default."'";
            break;
          case 'float' :
            $defaultStr = " DEFAULT '".(double)$default."'";
            break;
          default :
            $defaultStr = " DEFAULT '".$this->escapeStr($default)."'";
            break;
        }
      }
      switch (strtolower(trim($field['type']))) {
        case 'integer' :
          $size = ($field['size'] > 0) ? (int)$field['size'] : 1;
          if ($size <= 2) {
            $result = "SMALLINT";
          } elseif ($size <= 4) {
            $result = "INTEGER";
          } else {
            $result = "BIGINT";
          }
          break;
        case 'float' :
          list($before,$after) = explode(',',$field['size']);
          $before = ($before > 0) ? (int)$before : 1;
          $after = (int)$after;
          $result = "NUMERIC(".$before.','.$after.")";
          break;
        case 'string' :
        default :
          $size = ($field['size'] > 0) ? (int)$field['size'] : 1;
          if ($size <= 4) {
            $result = "CHAR(".$field['size'].")";
          } elseif ($size <= 255) {
            $result = "VARCHAR(".$field['size'].")";
          } else {
            $result = "TEXT";
          }
          break;
      }
      if ($seperated) {
        return array(
          'type' => $result,
          'default' => $defaultStr,
          'not_null' => $notNullStr,
          'autoinc' => FALSE
        );
      } else {
        return $result.$defaultStr.$notNullStr;
      }
    }
    return FALSE;
  }

  /**
  * Query table structure
  *
  * @param string $tableName
  * @param string $tablePrefix optional, default value ''
  * @access public
  * @return array
  */
  function queryTableStructure($tableName, $tablePrefix = '') {
    $fields = array();
    if ($tablePrefix) {
      $table = $tablePrefix.'_'.$tableName;
    } else {
      $table = $tableName;
    }
    $sql = "SELECT a.attname AS fieldname,
                t.typname AS fieldtype,
                pg_catalog.format_type(a.atttypid, a.atttypmod),
                a.attlen AS fieldsize,
                a.attnotNull AS notNull,
              (SELECT substring(d.adsrc for 128)
                FROM pg_catalog.pg_attrdef d
                WHERE d.adrelid = a.attrelid
                AND d.adnum = a.attnum
                AND a.atthasdef) AS defaultwert
              FROM pg_class c, pg_attribute a, pg_type t
            WHERE relkind = 'r'
              AND c.relname='$table'
              AND a.attnum > 0
              AND a.atttypid = t.oid
              AND a.attrelid = c.oid
            ORDER BY a.attnum";

    if ($res = $this->query($sql)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $fields[$row['fieldname']] = $this->parsePostgresFieldData($row);
      }
    }
    $keys = array();
    $sql = "SELECT ic.relname AS index_name, a.attname AS column_name,
                    i.indisunique AS unique_key, i.indisprimary AS primary_key
              FROM pg_class bc, pg_class ic, pg_index i, pg_attribute a
              WHERE bc.oid = i.indrelid AND ic.oid = i.indexrelid
                AND (i.indkey[0] = a.attnum OR
                    i.indkey[1] = a.attnum OR
                    i.indkey[2] = a.attnum OR
                    i.indkey[3] = a.attnum OR
                    i.indkey[4] = a.attnum OR
                    i.indkey[5] = a.attnum OR
                    i.indkey[6] = a.attnum OR
                    i.indkey[7] = a.attnum)
                AND a.attrelid = bc.oid
                AND bc.relname = '$table'";
    if ($res = $this->query($sql)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        if ($row['primary_key'] == 't') {
          $keyName = 'PRIMARY';
        } elseif (strpos($row['index_name'], $table) === 0) {
          $keyName = substr($row['index_name'], strlen($table)+1);
        } else {
          $keyName = $row['index_name'];
        }

        $keys[$keyName]['orgname'] = $row['index_name'];
        $keys[$keyName]['name'] = $keyName;

        $keys[$keyName]['unique'] = ($row['unique_key'] == 't') ? 'yes' : 'no';
        $keys[$keyName]['fields'][] = $row['column_name'];
        $keys[$keyName]['fulltext'] = 'no';
      }
    }
    return array(
      'name' => $tableName,
      'fields' => $fields,
      'keys' => $keys
      );
  }

  /**
  * Create given table
  *
  * @param string $tableData
  * @param string $tablePrefix
  * @access public
  * @return boolean
  */
  function createTable(&$tableData, $tablePrefix) {
    if (is_array($tableData['fields']) && trim($tableData['name']) != '') {
      if ($tablePrefix) {
        $table = $tablePrefix.'_'.trim($tableData['name']);
      } else {
        $table = trim($tableData['name']);
      }
      $sql = 'CREATE TABLE '.$this->escapeStr(strtolower($table)).' ('.LF;
      $autoIncrementField = FALSE;
      foreach ($tableData['fields'] as $field) {
        $sql .= '  '.strtolower($field['name']).' '.
          $this->getPostgresFieldType($field).",\n";
      }
      if (is_array($tableData['keys'])) {
        if (isset($tableData['keys']['PRIMARY'])) {
          $sql .= 'CONSTRAINT '.$this->escapeStr(strtolower($table)).'_primary_key PRIMARY KEY ('.implode(',', $tableData['keys']['PRIMARY']['fields'])."),\n";
        }
        foreach($tableData['keys'] as $keyName=>$key) {
          if ($keyName != 'PRIMARY' && isset($key['unique']) && $key['unique'] == 'yes') {
            $sql .= 'CONSTRAINT '.$this->escapeStr(strtolower($table)).'_'.$keyName.' UNIQUE ';
            $sql .= '('.implode(',',$key['fields'])."),\n";
          }
        }
      }
      $sql = substr($sql, 0, -2)."\n)\n";
      if ($this->query($sql) !== FALSE) {
        foreach($tableData['keys'] as $key) {
          $this->addIndex($table,$key);
        }
        return TRUE;
      }
    }
    return FALSE;
  }

  function getFieldInfo($table, $fieldName) {
    $sql = "SELECT a.attname AS fieldname,
                t.typname AS fieldtype,
                pg_catalog.format_type(a.atttypid, a.atttypmod),
                a.attlen AS fieldsize,
                a.attnotNull AS notNull,
              (SELECT substring(d.adsrc for 128)
                FROM pg_catalog.pg_attrdef d
                WHERE d.adrelid = a.attrelid
                AND d.adnum = a.attnum
                AND a.atthasdef) AS defaultwert
              FROM pg_class c, pg_attribute a, pg_type t
            WHERE relkind = 'r'
              AND c.relname='".$this->escapeStr($table)."'
              AND a.attname='".$this->escapeStr($fieldName)."'
              AND a.attnum > 0
              AND a.atttypid = t.oid
              AND a.attrelid = c.oid
            ORDER BY a.attnum";
    if ($res = $this->query($sql)) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        return $this->parsePostgresFieldData($row);
      }
    }
    return FALSE;
  }

  /**
  * Add Field
  *
  * @param string $table
  * @param array &$fieldData
  * @access public
  * @return booblean
  */
  function addField($table, &$fieldData) {
    return $this->changeField($table, $fieldData);
  }

  /**
  * Change Field
  *
  * @param string $table
  * @param array &$fieldData
  * @access public
  * @return booblean
  */
  function changeField($table, &$fieldData) {
    $result = TRUE;
    $tableName = $this->escapeStr($table);
    $fieldName = $this->escapeStr($fieldData['name']);

    $xmlType = $this->getPostgresFieldType($fieldData);

    //check field exists in db
    if ($databaseField = $this->getFieldInfo($table, $fieldData['name'])) {
      $dbType = $this->getPostgresFieldType($databaseField);
      if ($xmlType != $dbType) {
        $sqlData = $this->getPostgresFieldType($fieldData, TRUE);
        $sql = "ALTER TABLE $tableName ALTER COLUMN $fieldName TYPE ".$sqlData['type'].";\n";
        $sql .= "ALTER TABLE $tableName ALTER COLUMN $fieldName SET ".$sqlData['not_null'].";\n";
        if ($sqlData['autoinc']) {
          $sql .= "ALTER TABLE $tableName ALTER COLUMN $fieldName SET DEFAULT nextval('public.{$table}_{$fieldName}_seq'::text);\n";
        } else {
          $sql .= "ALTER TABLE $tableName ALTER COLUMN $fieldName SET ".$sqlData['default'].";\n";
        }
        return ($this->query($sql) !== FALSE);
      }
      return TRUE;
    } else {
      //create field
      $sql = "ALTER TABLE ".$this->escapeStr($table)." ADD COLUMN ".$this->escapeStr($fieldData['name'])." ".$xmlType;
      return ($this->query($sql) !== FALSE);
    }
  }

  /**
  * Drop field
  *
  * @param string $table
  * @param string $field fieldname
  * @access public
  * @return booblean
  */
  function dropField($table, $field) {
    $sql = "ALTER TABLE ".$this->escapeStr($table)." DROP COLUMN ".$this->escapeStr($field)."";
    return ($this->query($sql) !== FALSE);
  }


  function getIndexInfo($table, $key) {
    $result = FALSE;
    $tableName = $this->escapeStr($table);
    $keyName = ($key == 'PRIMARY') ? 'primary_key' : $this->escapeStr($key);

    $sql = "SELECT ic.relname, ic.relname AS index_name, a.attname AS column_name,
                    i.indisunique AS unique_key, i.indisprimary AS primary_key
              FROM pg_class bc, pg_class ic, pg_index i, pg_attribute a
              WHERE bc.oid = i.indrelid AND ic.oid = i.indexrelid
                AND (i.indkey[0] = a.attnum OR
                    i.indkey[1] = a.attnum OR
                    i.indkey[2] = a.attnum OR
                    i.indkey[3] = a.attnum OR
                    i.indkey[4] = a.attnum OR
                    i.indkey[5] = a.attnum OR
                    i.indkey[6] = a.attnum OR
                    i.indkey[7] = a.attnum)
                AND a.attrelid = bc.oid
                AND bc.relname = '".$tableName."'
                AND ic.relname = '".$tableName.'_'.$keyName."'";

    if ($res = $this->query($sql)) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        if ($row['primary_key'] == 't') {
          $keyName = 'PRIMARY';
        } elseif (strpos($row['index_name'], $table) === 0) {
          $keyName = substr($row['index_name'], strlen($table)+1);
        } else {
          $keyName = $row['index_name'];
        }

        $result['orgname'] = $row['index_name'];
        $result['name'] = $keyName;

        $result['unique'] = ($row['unique_key'] == 't') ? 'yes' : 'no';
        $result['fields'][] = $row['column_name'];
        $result['fulltext'] = 'no';
      }
    }

    return $result;
  }

  /**
  * Add index
  *
  * @param table $table
  * @param array &$index
  * @access public
  * @return boolean
  */
  function addIndex($table, &$index) {
    return $this->changeIndex($table, $index, FALSE);
  }

  /**
  * Change Index
  *
  * @param atring $table
  * @param array &$index
  * @param boolean $dropCurrent optional, default value TRUE
  * @access public
  * @return boolean
  */
  function changeIndex($table, &$index, $dropCurrent = TRUE) {
    $key = $this->getIndexInfo($table, $index['name']);

    if ($dropCurrent && $key) {
      $this->dropIndex($table, $index['name']);
    }

    if (isset($index['fields']) && is_array($index['fields'])) {
      $fields = '('.implode(',',$index['fields']).")";
      if ($index['name'] == 'PRIMARY') {
        $sql = "ALTER TABLE ".$this->escapeStr($table)." ADD PRIMARY KEY ".$fields;
      } elseif (isset($index['unique']) && $index['unique'] == 'yes') {
        $sql = 'CREATE UNIQUE INDEX '.$this->escapeStr($table).'_'.
          $this->escapeStr($index['name']).' ON '.$this->escapeStr($table).' '.$fields;
      } else {
        $sql = 'CREATE INDEX '.$this->escapeStr($table).'_'.$this->escapeStr($index['name']).' ON '.
          $this->escapeStr($table).' '.$fields;
      }
      return ($this->query($sql) !== FALSE);
    }
    return FALSE;
  }

  /**
  * Drop Index
  *
  * @param string $table
  * @param string $name
  * @access public
  * @return boolean
  */
  function dropIndex($table, $name) {
    if ($key = $this->getIndexInfo($table, $name)) {
      $keyName = $this->escapeStr($key['orgname']);
      if ($key['name'] == 'PRIMARY') {
        $sql = 'ALTER TABLE '.$tableName.' DROP CONSTRAINT '.$keyName;
      } else {
        $sql = 'DROP INDEX '.$keyName;
      }
      return ($this->query($sql) !== FALSE);
    }
    return FALSE;
  }

  function getSQLFunctionParams($params) {
    $result = '';
    for($i=0;$i<count($params);$i+=2) {
      $result .= $this->getSQLFunctionParam($params[$i], @$params[$i+1]).',';
    }
    return substr($result, 0, -1);
  }
  
  function getSQLFunctionParam($value, $escaping) {
    if (isset($escaping) && $escaping == FALSE) {
      return $value;
    } else {
      return "'".$this->escapeStr($value])."'";
    }
  }

  /**
  * DBMS spezific SQL source
  *
  * @param $function sql function
  * @param $params params
  * @access public
  * @return mixed sql string or FALSE
  */
  function getSQLSource($function, $params) {
    switch (strtoupper($function)) {
      case 'CONCAT' :
        $result = '';
        for($i=0;$i<count($params);$i+=2) {
          $result .= $this->getSQLFunctionParam($params[$i], @$params[$i+1]).' ||';
        }
        return substr($result, 0, -2).')';
      case 'SUBSTRING' :
        return ' SUBSTRING('.$this->getSQLFunctionParams($params).')';
      case 'LOWER' :
        return ' LOWER('.$this->getSQLFunctionParams($params).')';
      case 'UPPER' :
        return ' UPPER('.$this->getSQLFunctionParams($params).')';
      case 'LOCATE' :
        $result = ' POSITION(';
        if (count($params) >= 3) {
          $result .= $this->getSQLFunctionParam($params[0], @$params[1).' IN ';
          $result .= $this->getSQLFunctionParam($params[2], @$params[3).')';
        }
        return $result;
    }
    return FALSE;
  }

  function compareFieldStructure(&$xmlField, &$databaseField) {
    if (@$xmlField['type'] != @$databaseField['type']) {
      return TRUE;
    } elseif (@$xmlField['size'] != @$databaseField['size']) {
      if (@$xmlField['type'] == 'string' && @$xmlField['size'] > 255 && @$databaseField['size'] > 255) {
        return FALSE;
      }
      if (@$xmlField['type'] == 'integer' && @$databaseField['autoinc'] == 'yes' && @$xmlField['autoinc'] == 'yes' &&
          (($xmlField['size'] == 2 && $databaseField['size'] == 4) || ($xmlField['size'] == 6 && $databaseField['size'] == 8))) {
        return FALSE;
      }
      return TRUE;
    } elseif (@$xmlField['null'] != @$databaseField['null']) {
      return TRUE;
    } elseif (@$xmlField['autoinc'] == 'yes' && @$databaseField['autoinc'] != 'yes') {
      return TRUE;
    } elseif (@$xmlField['default'] != @$databaseField['default']) {
      return TRUE;
    }
    return FALSE;
  }

  function compareKeyStructure(&$xmlKey, &$databaseKey) {
    if ((@$xmlKey['unique'] == 'yes' || $xmlKey['name'] == "PRIMARY") !=
          (@$databaseKey['unique'] == 'yes' || $databaseKey['name'] == "PRIMARY")) {
      return TRUE;
    } elseif (@count(array_intersect($xmlKey['fields'], $databaseKey['fields'])) != @count($xmlKey['fields'])) {
      return TRUE;
    }
    return FALSE;
  }
}

/**
* DB-Abstractionslayer - result object PostgreSQL
* @package papaya_base
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class dbresult_pgsql extends dbresult_base {

  /**
  * destructor
  *
  * Free memory, unset self and resultID
  */
  function free() {
    if (@is_resource($this->result)) {
      pg_free_result($this->result);
      unset($this->result);
      unset($this);
    }
  }

  /**
  * Fetch next row of result
  *
  * @param integer $mode line return modus
  * @return mixed FALSE or next line
  */
  function fetchRow($mode = DB_FETCHMODE_DEFAULT) {
    if (@is_resource($this->result)) {
      if ($mode == DB_FETCHMODE_ASSOC) {
        $result = pg_fetch_assoc($this->result);
      } else {
        $result = pg_fetch_row($this->result);
      }
      if (@is_array($result)) {
        $this->recNo++;
      }
      return $result;
    }
    return FALSE;
  }

  /** helpme
  * Return absolute data number of query
  *
  * @return mixed number or FALSE
  */
  function absCount() {
    if (($i = $this->count()) < $this->_absCount) {
      return $this->_absCount;
    } else {
      return $i;
    }
    return FALSE;
  }

  /**
  * Number rows affected by query
  * @return mixed number of rows or FALSE
  */
  function count() {
    if (is_resource($this->result)) {
      return pg_num_rows($this->result);
    }
    return FALSE;
  }

  /**
  * Search index
  *
  * Move record pointer to given index
  * next call of pg_fetch_row() returns wanted value
  *
  * @param $index
  * @return boolean
  */
  function seek($index) {
    if (@is_resource($this->result)) {
      if (pg_data_seek($this->result, $index)) {
        $this->recNo = $index;
        return TRUE;
      }
    }
    return FALSE;
  }
}
?>