<?php
/**
* SQLite database access classes
*
* @package papaya_base
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: sqlite.php,v 1.11 2006/06/06 11:35:42 weinert Exp $
*/

/**
* basic database connection and result class
*/
require_once(dirname(__FILE__).'/base.php');

/**
* DB-abstraction layer - SQLite
*
* @package papaya_base
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class dbcon_sqlite extends dbcon_base {

  var $connectErrorString = '';

  var $callbacks = array();

  function extensionFound() {
    return (bool)extension_loaded('sqlite');
  }
  
  /**
  * Establish connection to database
  *
  * @return resource $this->databaseConnection connection ID
  */
  function connect() {
    if (!(isset($this->databaseConnection) && is_resource($this->databaseConnection))) {
      $database = $this->databaseConfiguration['database'];
      $error = '';
      $this->databaseConnection = @sqlite_open($database, 0666, $error);
      if (isset($this->databaseConnection) && is_resource($this->databaseConnection)) {
        $this->connectErrorString = '';
        return TRUE;
      } else {
        unset($this->databaseConnection);
        $this->connectErrorString = $error;
        return FALSE;
      }
    } else {
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Provides error messages of last database connection
  *
  * @return array error number, error message and last sql query
  */
  function getError() {
    if (isset($this->databaseConnection) && is_resource($this->databaseConnection)) {
      $code = @sqlite_last_error($this->databaseConnection);
      return array('no'=>$code, 'str'=>@sqlite_error_string($code), 'lastquery'=>$this->lastSQLQuery);
    } elseif (isset($this->connectErrorString) && $this->connectErrorString != '') {
      return array('no'=>NULL, 'str'=> $this->connectErrorString, '');
    } else {
      return array('no'=>NULL, 'str'=> NULL, 'lastquery'=>$this->lastSQLQuery);
    }
  }

  /**
  * Return an explanation of the last SELECT-SQL-query
  *
  * Explan the last query, number of lines etc.
  *
  * @return mixed Result or FALSE, if last request wasn't a SELECT query
  */
  function getExplain() {
    if (preg_match('/^SELECT\s/i', $this->lastSQLQuery)) {
      if ($res = @sqlite_query($this->databaseConnection, 'EXPLAIN  '.$this->lastSQLQuery)) {
        $result = '<table width="100%" border=="1">';
        $result .= '<tr><th>addr</th><th>opcode</th><th>param1</th><th>param2</th><th>param3</th></tr>';
        while($row = @sqlite_fetch_array($res, SQLITE_NUM)) {
          $result .= '<tr>';
          foreach($row as $val) {
            $result .= '<td>';
            $result .= (isset($val)) ? papaya_strings::escapeHTMLChars($val) : '';
            $result .= '</td>';
          }
          $result .= '</tr>';
        }
        $result .= '</table>';
        return $result;
      }
    }
    return FALSE;
  }


  /**
  * String ecsaping for PostgreSQL use
  *
  * @param mixed $value Value to escape
  * @return string escaped value.
  */
  function escapeStr($value) {
    return @sqlite_escape_string($value);
  }

  /**
  * Execute PostgreSQL-query
  *
  * @param string $sql SQL-String with query
  * @param integer $max maximum number of returned records
  * @param integer $offset Offset
  * @param boolean $debug Debug Query?
  * @return mixed FALSE or number of affected_rows or database result object
  */
  function &query($sql, $max=NULL, $offset=NULL, $debug = FALSE) {
    if (@get_class($this->lastResult) == 'dbresult_sqlite') {
      $this->lastResult->free();
    }
    $absCount = 0;
    if (isset($max) && strpos(trim($sql), 'SELECT') === 0) {
      if (preg_match('~^\s*(SELECT.*)(\bFROM\b.*?)(\bORDER\s+BY.*)?$~si', $sql, $regs)) {
        $countSql = 'SELECT COUNT(*) '.$regs[2];
        if ($res = @sqlite_query($countSql, $this->databaseConnection)) {
          if ($row = @sqlite_fetch_array($res, SQLITE_NUM)) {
            $absCount = $row[0];
          }
        }
      }
      $sql .= (isset($offset)) ? ' LIMIT '.(int)$offset.','.(int)$max : ' LIMIT '.(int)$max;      
    }

    $this->lastSQLQuery = $sql;
    $res = @sqlite_query($sql, $this->databaseConnection);
    if ($res) {
      if (@is_resource($res)) {
        $this->lastResult = &new dbresult_sqlite($res);
        $this->lastResult->setLimit($max, $offset);
        $this->lastResult->_absCount = $absCount;
        return $this->lastResult;
      } else {
        $result = @sqlite_changes($this->databaseConnection);
        return $result;
      }
    } else {
      $result = FALSE;
      return $result;
    }
  }

  /**
  * Insert record into table
  *
  * @param string $table table
  * @param string $idField primary key value
  * @param array $values insert values
  * @return mixed FALSE or Id of new record
  */
  function insertRecord($table, $idField, $values = NULL) {
    if (isset($idField)) {
      $values[$idField] = NULL;
    }
    if (isset($values) && is_array($values) && count($values) > 0) {
      $fieldString = '';
      $valueString = '';
      foreach($values as $field=>$value) {
        if ($value !== NULL) {
          $fieldString .= $this->escapeStr($field).', ';
          $valueString .= "'".$this->escapeStr($value)."', ";
        }
      }
      $fieldString = substr($fieldString, 0, -2);
      $valueString = substr($valueString, 0, -2);
      $sql = 'INSERT INTO '.$this->escapeStr($table).' ('.$fieldString.') VALUES ('.$valueString.')';
      if ($this->query($sql)) {
        if (isset($idField)) {
          $newId = sqlite_last_insert_rowid($this->databaseConnection);
          return $newId;
        } else {
          return sqlite_changes($this->databaseConnection);
        }
      }
    }
    return FALSE;
  }


  function insertRecords($table, $values) {
    $lastFields = NULL;
    $specialChars = array("\t"=>'\t', "\r"=>'\r', "\n"=>'\n');
    if (isset($values) && is_array($values) && count($values) > 0) {
      foreach($values as $data) {
        if (is_array($data) && count($data) > 0) {
          if (!$this->insertRecord($table, NULL, $data)) {
            return FALSE;
          }
        }
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Update records via filter
  *
  * @param string $table table name
  * @param array $values update values
  * @param string $filter Filter string without WHERE condition
  * @return mixed FALSE or number of affected_rows or database result object
  */
  function updateRecord($table, $values, $filter) {
    if (isset($values) && is_array($values) && count($values) > 0) {
      $sql = FALSE;
      foreach($values as $col=>$val) {
        $fieldName = trim($col);
        $sql .= " $fieldName='".$this->escapeStr($val)."', ";
      }
      if ($sql) {
        $sql = "UPDATE $table SET ".substr($sql, 0, -2)." WHERE ".$this->getSQLCondition($filter);
        return $this->query($sql);
      } else {
        $this->lastSQLQuery = 'NO DATA';
      }
    } else {
      $this->lastSQLQuery = '';
    }
    return FALSE;
  }

  /**
  * Delete records by filter
  *
  * @param string $table table name
  * @param string $filter Filter string without WHERE condition
  * @return mixed FALSE or number of affected_rows or database result object
  */
  function deleteRecord($table, $filter) {
    $sql = "DELETE FROM $table WHERE ".$this->getSQLCondition($filter);
    return $this->query($sql);
  }

  /**
  * Get table names
  *
  * @access public
  * @return array
  */
  function queryTableNames() {
    $sql = "SELECT name FROM sqlite_master WHERE type = 'table'";
    $result = array();
    if ($res = $this->query($sql)) {
      while($row = $res->fetchRow()) {
        $result[] = $row[0];
      }
    }
    return $result;
  }

  /**
  * Parse PostgreSQL field data
  *
  * @param array &$row
  * @access private
  * @return array
  */
  function parseSQLiteFieldData(&$row) {
    $size = 0;
    $autoIncrement = FALSE;
    $default = NULL;

    switch ($row['type']) {
      case 'BIGSERIAL' :
      case 'BIGINT' :
        $type = 'integer';
        $size = 8;
        break;
      case 'SERIAL' :
      case 'INT' :
      case 'INTEGER' :
        $type = 'integer';
        $size = 4;
        if ($row['pk'] == 1) {
          $autoIncrement = TRUE;
        }
        break;
      case 'SMALLINT' :
        $type = 'integer';
        $size = 2;
        break;
      case 'TEXT' :
        $type = 'string';
        $size = 65535;
        break;
      default :
        if (preg_match('~VARCHAR\((\d+)\)~', $row['type'], $regs)) {
          $type = 'string';
          $size = (int)$regs[1];
        } elseif (preg_match('~CHAR\((\d+)\)~', $row['type'], $regs)) {
          $type = 'string';
          $size = (int)$regs[1];
        } else {
          $type = 'string';
          $size = 16777215;
        }
    }

    if ($autoIncrement) {
      $notNull = TRUE;
    } elseif ($row['not_null'] != 0) {
      $notNull = TRUE;
      if ($type != 'integer' || $row['dflt_value'] != 0)
        $default = $row['dflt_value'];
    } else {
      $notNull = FALSE;
    }

    $result = array(
      'name' => $row['name'],
      'type' => $type,
      'size' => $size,
      'null' => $notNull ? 'no' : 'yes',
      'autoinc' => $autoIncrement ? 'yes' : 'no',
      'default' => $default
    );

    return $result;
  }

  /**
  * PostgresSQL field type
  *
  * @param string $type
  * @param string $size
  * @access private
  * @return string
  */
  function getSQLiteFieldType(&$field, $primaryKey = FALSE) {
    if (isset($field['autoinc']) && $field['autoinc'] == 'yes') {
      return 'INTEGER PRIMARY KEY';
    } else {
      if (isset($field['null']) && $field['null'] == 'yes') {
        $default = NULL;
        $notNullStr = '';
      } else {
        $default = '';
        $notNullStr = ' NOT NULL';
      }
      if (isset($field['default'])) {
        $default = $field['default'];
      }
      $defaultStr = '';
      if (isset($default)) {
        switch(strtolower($field['type'])) {
          case 'integer' :
            $defaultStr = " DEFAULT '".(int)$default."'";
            break;
          case 'float' :
            $defaultStr = " DEFAULT '".(double)$default."'";
            break;
          default :
            $defaultStr = " DEFAULT '".$this->escapeStr($default)."'";
            break;
        }
      }
      switch (strtolower(trim($field['type']))) {
        case 'integer' :
          $size = ($field['size'] > 0) ? (int)$field['size'] : 1;
          if ($size <= 2) {
            $result = "SMALLINT";
          } elseif ($size <= 4) {
            $result = "INTEGER";
          } else {
            $result = "BIGINT";
          }
          break;
        case 'float' :
          list($before,$after) = explode(',',$field['size']);
          $before = ($before > 0) ? (int)$before : 1;
          $after = (int)$after;
          $result = "NUMERIC(".$before.','.$after.")";
          break;
        case 'string' :
        default :
          $size = ($field['size'] > 0) ? (int)$field['size'] : 1;
          if ($size <= 4) {
            $result = "CHAR(".$field['size'].")";
          } elseif ($size <= 255) {
            $result = "VARCHAR(".$field['size'].")";
          } else {
            $result = "TEXT";
          }
          break;
      }
      if ($primaryKey) {
        $result .= ' PRIMARY KEY';
      }
      return $result.$defaultStr.$notNullStr;
    }
    return FALSE;
  }

  /**
  * Query table structure
  *
  * @param string $tableName
  * @param string $tablePrefix optional, default value ''
  * @access public
  * @return array
  */
  function queryTableStructure($tableName, $tablePrefix = '') {
    $fields = array();
    $keys = array();
    if ($tablePrefix) {
      $table = $tablePrefix.'_'.$tableName;
    } else {
      $table = $tableName;
    }
    $sql = "PRAGMA table_info('".$this->escapeStr($table)."');";
    if ($res = $this->query($sql)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $fields[$row['name']] = $this->parseSQLiteFieldData($row);
        if ($row['pk'] == 1) {
          if (isset($keys['PRIMARY']) && is_array($keys['PRIMARY'])) {
            $keys['PRIMARY']['fields'][] = $row['name'];
          } else {
            $keys['PRIMARY']['orgname'] = 'PRIMARY';
            $keys['PRIMARY']['name'] = 'PRIMARY';
            $keys['PRIMARY']['unique'] = 'yes';
            $keys['PRIMARY']['fields'] = array($row['name']);
            $keys['PRIMARY']['fulltext'] = 'no';
            $keys['PRIMARY']['autoinc'] = ($row['type'] == 'INTEGER') ? 'yes' : 'no';
          }
        }
      }
    }
    $sql = "PRAGMA index_list('".$this->escapeStr($table)."')";
    if ($res = $this->query($sql)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        if (preg_match('/\('.preg_quote($table).' autoindex (\d+)\)/', $row['name'])) {
          $keyName = 'PRIMARY';
        } elseif (strpos($row['name'], $table) === 0) {
          $keyName = substr($row['name'], strlen($table)+1);
        } else {
          $keyName = $row['name'];
        }

        $keys[$keyName]['orgname'] = $row['name'];
        $keys[$keyName]['name'] = $keyName;
        $keys[$keyName]['unique'] = ($row['unique'] == '1') ? 'yes' : 'no';
        $keys[$keyName]['fields'] = array();
        $keys[$keyName]['fulltext'] = 'no';
      }
      foreach($keys as $keyName=>$keyData) {
        $sql = "PRAGMA index_info('".$this->escapeStr($keyData['orgname'])."')";
        if ($res = $this->query($sql)) {
          while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
            $keys[$keyName]['fields'][] = $row['name'];
          }
        }
      }
    }
    return array(
      'name' => $tableName,
      'fields' => $fields,
      'keys' => $keys
    );
  }

  /**
  * Create given table
  *
  * @param string $tableData
  * @param string $tablePrefix
  * @access public
  * @return boolean
  */
  function createTable(&$tableData, $tablePrefix) {
    if (is_array($tableData['fields']) && trim($tableData['name']) != '') {
      if ($tablePrefix) {
        $table = $tablePrefix.'_'.trim($tableData['name']);
      } else {
        $table = trim($tableData['name']);
      }
      if (isset($tableData['keys']) && isset($tableData['keys']['PRIMARY']) &&
          isset($tableData['keys']['PRIMARY']['fields']) && is_array($tableData['keys']['PRIMARY']['fields']) &&
          count($tableData['keys']['PRIMARY']['fields']) == 1) {
        $primaryKeyField = &$tableData['keys']['PRIMARY']['fields'][0];
      } else {
        $primaryKeyField = '';
      }
      $sql = 'CREATE TABLE '.$this->escapeStr(strtolower($table)).' ('.LF;
      foreach ($tableData['fields'] as $field) {
        $sql .= '  '.strtolower($field['name']).' '.
          $this->getSQLiteFieldType($field, $field['name'] == $primaryKeyField).",\n";
      }
      if (is_array($tableData['keys'])) {
        foreach($tableData['keys'] as $keyName=>$key) {
          if ($keyName == 'PRIMARY' && is_array($key['fields']) && count($key['fields']) > 1) {
            $sql .= 'PRIMARY KEY ('.implode(',',$key['fields'])."),\n";
          } elseif ($keyName != 'PRIMARY' && isset($key['unique']) && $key['unique'] == 'yes') {
            $sql .= 'CONSTRAINT '.$this->escapeStr(strtolower($table)).'_'.$keyName.' UNIQUE ';
            $sql .= '('.implode(',',$key['fields'])."),\n";
          }
        }
      }
      $sql = substr($sql, 0, -2)."\n)\n";
      if ($this->query($sql) !== FALSE) {
        foreach($tableData['keys'] as $key) {
          $this->addIndex($table,$key);
        }
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  * Add Field
  *
  * @param string $table
  * @param array &$fieldData
  * @access public
  * @return booblean
  */
  function addField($table, &$fieldData) {
    return $this->changeField($table, $fieldData);
  }

  /**
  * Change Field
  *
  * @param string $table
  * @param array &$fieldData
  * @access public
  * @return booblean
  */
  function changeField($table, &$fieldData) {
    return FALSE;
  }

  /**
  * Drop field
  *
  * @param string $table
  * @param string $field fieldname
  * @access public
  * @return booblean
  */
  function dropField($table, $field) {
    return FALSE;
  }

  function getIndexInfo($table, $key) {
    $result = FALSE;
    $tableName = $this->escapeStr($table);
    if ($key == 'PRIMARY') {
      $sql = "PRAGMA table_info('".$this->escapeStr($table)."');";
      if ($res = $this->query($sql)) {
        while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          if ($row['pk'] == 1) {
            if (!is_array($result)) {
              $result['name'] = 'PRIMARY';
              $result['unique'] = 1;
              $result['fields'] = array($row['name']);
              $result['fulltext'] = 'no';
              $result['autoinc'] = ($row['type'] == 'INTEGER') ? 'yes' : 'no';
            } else {
              $result['fields'][] = $row['name'];
            }
          }
        }
      }
    } else {
      $keyName = $this->escapeStr($key);
      $sql = "PRAGMA index_list('".$this->escapeStr($table)."')";
      if ($res = $this->query($sql)) {
        while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          if (strpos($row['name'], $table) === 0 && $keyName == substr($row['name'], strlen($table)+1)) {
            $result['orgname'] = $row['name'];
            $result['name'] = $keyName;
            $result['unique'] = ($row['unique'] == '1') ? 'yes' : 'no';
            $result['fields'] = array();
            $result['fulltext'] = 'no';
            $sql = "PRAGMA index_info('".$this->escapeStr($keyData['orgname'])."')";
            if ($res = $this->query($sql)) {
              while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
                $result['fields'][] = $row['name'];
              }
            }
          } else {
            continue;
          }
        }
      }
    }
    return $result;
  }

  /**
  * Add index
  *
  * @param table $table
  * @param array &$index
  * @access public
  * @return boolean
  */
  function addIndex($table, &$index) {
    return $this->changeIndex($table, $index, FALSE);
  }

  /**
  * Change Index
  *
  * @param atring $table
  * @param array &$index
  * @param boolean $dropCurrent optional, default value TRUE
  * @access public
  * @return boolean
  */
  function changeIndex($table, &$index, $dropCurrent = TRUE) {
    $key = $this->getIndexInfo($table, $index['name']);

    if ($dropCurrent && $key) {
      $this->dropIndex($table, $index['name']);
    }

    if (isset($index['fields']) && is_array($index['fields'])) {
      $fields = '('.implode(',',$index['fields']).")";
      if ($index['name'] == 'PRIMARY') {
        $sql = "ALTER TABLE ".$this->escapeStr($table)." ADD PRIMARY KEY ".$fields;
      } elseif (isset($index['unique']) && $index['unique'] == 'yes') {
        $sql = 'CREATE UNIQUE INDEX '.$this->escapeStr($table).'_'.
          $this->escapeStr($index['name']).' ON '.$this->escapeStr($table).' '.$fields;
      } else {
        $sql = 'CREATE INDEX '.$this->escapeStr($table).'_'.$this->escapeStr($index['name']).' ON '.
          $this->escapeStr($table).' '.$fields;
      }
      return ($this->query($sql) !== FALSE);
    }
    return FALSE;
  }

  /**
  * Drop Index
  *
  * @param string $table
  * @param string $name
  * @access public
  * @return boolean
  */
  function dropIndex($table, $name) {
    return FALSE;
  }

  function getSQLFunctionParams($params) {
    $result = '';
    for($i=0;$i<count($params);$i+=2) {
      if (isset($params[$i+1]) && $params[$i+1] == FALSE) {
        $result .= $params[$i].',';
      } else {
        $result .= "'".$this->escapeStr($params[$i])."',";
      }
    }
    return substr($result, 0, -1);
  }

  /**
  * DBMS spezific SQL source
  *
  * @param $function sql function
  * @param $params params
  * @access public
  * @return mixed sql string or FALSE
  */
  function getSQLSource($function, $params) {
    switch (strtoupper($function)) {
      case 'CONCAT' :
        $result = '';
        for($i=0;$i<count($params);$i+=2) {
          if (isset($params[$i+1]) && $params[$i+1] = FALSE) {
            $result .= $params[$i].', ||';
          } else {
            $result .= "'".$this->escapeStr($params[$i])."' ||";
          }
        }
        return substr($result, 0, -2).')';
      case 'SUBSTRING' :
        return ' SUBSTRING('.$this->getSQLFunctionParams($params).')';
      case 'LOWER' :
        return ' LOWER('.$this->getSQLFunctionParams($params).')';
      case 'UPPER' :
        return ' UPPER('.$this->getSQLFunctionParams($params).')';
      case 'LOCATE' :
        if (!isset($this->callbacks['LOCATE'])) {
          sqlite_create_function($this->databaseConnection, 'LOCATE', array(&$this, 'sqliteCallbackLOCATE'));
          $this->callbacks['LOCATE'] = TRUE;
        }
        return ' LOCATE('.$this->getSQLFunctionParams($params).')';
    }
    return FALSE;
  }

  function sqliteCallbackLOCATE($needle, $haystack, $offset = 0) {
    $pos= strpos($haystack, $needle, $offset);
    if ($pos !== FALSE) {
      return ++$pos;
    } else {
      return 0;
    }
  }

  function compareFieldStructure(&$xmlField, &$databaseField) {
    if (@$xmlField['type'] != @$databaseField['type']) {
      return TRUE;
    } elseif (@$xmlField['size'] != @$databaseField['size']) {
      if (@$xmlField['type'] == 'string' && @$xmlField['size'] > 255 && @$databaseField['size'] > 255) {
        return FALSE;
      }
      if (@$xmlField['type'] == 'integer' && @$databaseField['autoinc'] == 'yes' && @$xmlField['autoinc'] == 'yes') {
        return FALSE;
      }
      return TRUE;
    } elseif (@$xmlField['null'] != @$databaseField['null']) {
      return TRUE;
    } elseif (@$xmlField['autoinc'] == 'yes' && @$databaseField['autoinc'] != 'yes') {
      return TRUE;
    } elseif (@$xmlField['default'] != @$databaseField['default']) {
      return TRUE;
    }
    return FALSE;
  }

  function compareKeyStructure(&$xmlKey, &$databaseKey) {
    if ((@$xmlKey['unique'] == 'yes' || $xmlKey['name'] == "PRIMARY") !=
          (@$databaseKey['unique'] == 'yes' || $databaseKey['name'] == "PRIMARY")) {
      return TRUE;
    } elseif (@count(array_intersect($xmlKey['fields'], $databaseKey['fields'])) != @count($xmlKey['fields'])) {
      return TRUE;
    }
    return FALSE;
  }
}

/**
* DB-Abstractionslayer - result object - SQLite
* @package papaya_base
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class dbresult_sqlite extends dbresult_base {

  /**
  * destructor
  *
  * Free memory, unset self and resultID
  */
  function free() {
    if (@is_resource($this->result)) {
      unset($this->result);
      unset($this);
    }
  }

  /**
  * Fetch next row of result
  *
  * @param integer $mode line return modus
  * @return mixed FALSE or next line
  */
  function fetchRow($mode = DB_FETCHMODE_DEFAULT) {
    if (@is_resource($this->result)) {
      if ($mode == DB_FETCHMODE_ASSOC) {
        $result = sqlite_fetch_array($this->result, SQLITE_ASSOC);
        if (isset($result) && is_array($result)) {
          foreach($result as $key=>$val) {
            if (strpos($key, '.') !== FALSE) {
              $field = substr($key, strpos($key, '.')+1);
            } else {
              $field = $key;
            }
            $data[$field] = $val;
          }
          $result = $data;
        }
      } else {
        $result = sqlite_fetch_array($this->result, SQLITE_NUM);
      }
      if (@is_array($result)) {
        $this->recNo++;
      }
      return $result;
    }
    return FALSE;
  }

  /** helpme
  * Return absolute data number of query
  *
  * @return mixed number or FALSE
  */
  function absCount() {
    if (($i = $this->count()) < $this->_absCount) {
      return $this->_absCount;
    } else {
      return $i;
    }
    return FALSE;
  }

  /**
  * Number rows affected by query
  * @return mixed number of rows or FALSE
  */
  function count() {
    if (is_resource($this->result)) {
      return sqlite_num_rows($this->result);
    }
    return FALSE;
  }

  /**
  * Search index
  *
  * Move record pointer to given index
  * next call of pg_fetch_row() returns wanted value
  *
  * @param $index
  * @return boolean
  */
  function seek($index) {
    if (@is_resource($this->result)) {
      if (sqlite_seek($this->result, $index)) {
        $this->recNo = $index;
        return TRUE;
      }
    }
    return FALSE;
  }
}
?>